@@ -10,9 +10,9 @@
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
 #
-# import os
-# import sys
-# sys.path.insert(0, os.path.abspath('.'))
+import os
+import sys
+sys.path.append(os.path.abspath('sphinxext'))
 import datetime
 
 # -- Project information -----------------------------------------------------
@@ -34,9 +34,11 @@ extensions = [
     'sphinx.ext.viewcode',
     'sphinx.ext.autosummary',
     'sphinx.ext.imgmath',
-    'sphinxcontrib.fulltoc',
+    'sphinx_toolbox.collapse',
+    'fulltoc',
     'numpydoc',
-    'sphinx.ext.autosectionlabel'
+    'sphinx.ext.autosectionlabel',
+    'sphinx_copybutton',
 ]
 
 # Add any paths that contain templates here, relative to this directory.
@@ -79,6 +81,8 @@ html_theme_options = {
     "visitedlinkcolor": '#404040',
     "codebgcolor": '#fff1e6',
     "headbgcolor": '#ffd4b3',
+    "globaltoc_maxdepth": 1,
+    # "collapsiblesidebar": "True",
 }
 
 # Add any paths that contain custom themes here, relative to this directory.
@@ -86,7 +90,7 @@ html_theme_options = {
 
 
 # A shorter title for the navigation bar.  Default is the same as html_title.
-# html_short_title = None
+html_short_title = project + ' ' + version
 
 # The name of an image file (relative to this directory) to place at the top
 # of the sidebar.
