@@ -13,8 +13,8 @@ p = js.grace()
 p.title('CONTIN analysis of DLS data')
 p.multi(2, 1)
 p[0].plot(dls, legend='correlation data')
-p[0].plot(dls.distributions.X, dls.distributions.Y / 10, li=1, legend='relax. time intensity weighted')
-p[1].plot(dls.distributions[1], dls.distributions[2] / 10, li=1, legend='radius intensity weighted')
+p[0].plot(dls.distributions.X, dls.distributions.Y / 20, li=1, legend='relax. time intensity weighted')
+p[1].plot(dls.distributions[1], dls.distributions[2] / 20, li=1, legend='radius intensity weighted')
 # p[2].plot(dls.distributions[0],dls.distributions[2]/1000,li=1,legend='radius time intensity weighted')
 
 p[0].xaxis(scale='l', label=r'correlation time / µs ', tick=[10, 9])
@@ -24,7 +24,7 @@ p[1].xaxis(scale='l', label=r'radius / nm ', tick=[10, 9])
 # do the contin with the .X .Y data as intensity weighted relaxation time
 resx = js.dls.contin(dls, a=173, tmin=1, tmax=1e5, typ=0, bgr=0.05, distribution='x', T=(273 + dls.Temperature))
 p[0].plot(resx[0].contin_result_fit, li=4, sy=0, legend='contin result fit relaxation time')
-p[0].plot(resx[0].contin_bestFit.X, resx[0].contin_bestFit.Y * 100, sy=[1, 0.3, 4], legend='relax. time distribution')
+p[0].plot(resx[0].contin_bestFit.X, resx[0].contin_bestFit.Y * 10, sy=[1, 0.3, 4], legend='relax. time distribution')
 
 # determine the peak areas of the two peaks
 ff = resx[0].contin_bestFit
@@ -35,13 +35,6 @@ fp2 = np.trapz(ff.Y[p2], ff.X[p2])
 p[0].text('fraction %.2g' % fp1, 90, 1.5, color=4)
 p[0].text('fraction %.2g' % fp2, 10000, 0.5, color=4)
 
-# calc weight for .X intervals
-weight = ff.X.copy()
-weight[1:-1] = (ff.X[2:] - ff.X[:-2]) / 2
-weight[0] = ff.X[1] - ff.X[0]
-weight[-1] = ff.X[-2] - ff.X[-1]
-p[0].plot(resx[0].contin_bestFit.X, resx[0].contin_bestFit.Y * 10 * weight, sy=[2, 0.3, 8],
-          legend='relax. time distribution*weight')
 
 # do the contin with the .X .Y data as intensity weighted relaxation time
 resd = js.dls.contin(dls, a=173, tmin=1, tmax=1e5, typ=0, bgr=0.05, distribution='d', T=(273 + dls.Temperature))
