@@ -19,11 +19,11 @@
 #
 """
 This is a rudimentary interface to `matplotlib <https://matplotlib.org/>`_ to use dataArrays/sasImage easier.
-The standard way to use matplotlib is full available without using this module and recomended for more
-complicated use cases. Nevertheless the source can be used as template to be adapted.
+The standard way to use matplotlib is full available without using this module and recommended for more
+complicated use cases. Nevertheless, the source can be used as template to be adapted.
 
 The intention is to allow fast/easy plotting (one command to plot) with some convenience
-function in relation to dataArrays and in a non blocking mode of matplotlib.
+function in relation to dataArrays and in a non-blocking mode of matplotlib.
 E.g. to include automatically the value of an attribute (qq in example) in the legend for daily work
 and inspection of your data ::
 
@@ -54,7 +54,7 @@ Example 1::
     xyz.setColumnIndex(ix=0,iy=2,iz=1)
 
     # plot it
-    p=js.mplot(20,10)
+    p=js.mplot(20,10)  # with size in cm
     p.Multi(1,2)
     p[0].Plot(i5,sy=[-1,0.2,-1,-1],li=1,legend='Q= $q')
     js.mpl.contourImage(xyz,invert_yaxis=True,ax=p[1])
@@ -65,13 +65,21 @@ Example 1::
     p[0].Legend(x=1.43,y=1,charsize=0.8) # x,y in relative units of the plot
     p[0].Yaxis(label='I(Q,t)/I(Q,0)',min=0.01)
     p[0].Xaxis(label='Q / 1/nm',max=120)
-    # p.savefig(js.examples.imagepath+'/lineandcontourImage.jpg')
+
+    # p.savefig(js.examples.imagepath+'/lineandcontourImage.jpg', dpi=100)
 
 .. image:: ../../examples/images/lineandcontourImage.jpg
     :align: center
     :width: 50 %
     :alt: lines and contourImage
 
+Save in given size and resolution, e.g. 3.25 inch JACS column width with 600 dpi ::
+
+    # the factor 3 is only to get a reasonable screen size with reasonable char sizes
+    p.set_size_inches(3.25*3, 1.6*3)
+    p.savefig('lineandcontourImage.jpg', dpi=200)
+
+
 See another Example :py:func:`~.test`
 
 **Some short hints for matplotlib**
@@ -604,7 +612,7 @@ class jsFigure(matplotlib.figure.Figure):
         """
         Create figure with Axes as jspaperAxes projection.
 
-        This i used from mplot and should not be called directly.
+        This is used from mplot and should not be called directly.
 
         Examples
         --------
@@ -623,9 +631,12 @@ class jsFigure(matplotlib.figure.Figure):
 
         """
         self.headless = kwargs.pop('headless', _headless)
+        size = kwargs.pop('size', None)
         for opt, val in zip(['facecolor', 'frameon', 'facecolor', 'edgecolor'], ['w', False, 'w', 'w']):
             if opt not in kwargs:
                 kwargs[opt] = val
+        if size:
+            kwargs.update(figsize=size)
         matplotlib.figure.Figure.__init__(self, *args, **kwargs)
         self.add_subplot(1, 1, 1, projection='paper')
 
@@ -691,8 +702,13 @@ class jsFigure(matplotlib.figure.Figure):
     def Save(self, filename, format=None, dpi=None, **kwargs):
         """
         Save with filename
-        """
 
+        Same options as matplotlib savefig.
+
+        """
+        size = kwargs.pop('size', None)
+        if size and dpi:
+            self.set_size_inches(size[0], size[1])
         self.savefig(filename, format=format, dpi=dpi, **kwargs)
 
     def is_open(self):
@@ -1019,7 +1035,7 @@ def _isregularspaced(sequence):
 
 
 def contourImage(x, y=None, z=None, levels=None, fontsize=10, colorMap='jet', scale='norm', lineMap=None,
-                 axis=None, origin=None, block=False, invert_yaxis=False, invert_xaxis=False,
+                 axis=None, origin='upper', block=False, invert_yaxis=False, invert_xaxis=False,
                  linthresh=1, linscale=1, badcolor=None, ax=None):
     """
     Image with contour lines of 3D dataArrays or sasImage/image array.
@@ -1932,6 +1948,10 @@ def test(keepopen=True):
      fig[0].tick_params(labelbottom=False)
      fig[1].Xaxis(label='x-axis')
 
+     # plot with a given size and dpi (3.25 inch JACS column width)
+     fig.set_size_inches(3.25*2, 2*2)
+     fig.savefig("sample.png", dpi=100)  # with 300dpi results in 600dpi
+
     """
 
     import jscatter as js
