@@ -2,7 +2,7 @@
 # written by Ralf Biehl at the Forschungszentrum Jülich ,
 # Jülich Center for Neutron Science 1 and Institute of Complex Systems 1
 #    Jscatter is a program to read, analyse and plot data
-#    Copyright (C) 2015-2019  Ralf Biehl
+#    Copyright (C) 2015-2021  Ralf Biehl
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@ import unittest
 import os
 import numpy.testing as nptest
 import warnings
+from numpydoc.docscrape import FunctionDoc
+import inspect
 
 import numpy as np
 import scipy
@@ -51,6 +53,7 @@ from scipy.spatial.transform import Rotation as Rot
 import jscatter as js
 from .libs import test_imagehash
 
+
 asciitext = """# these are really ugly data
  # temp     ;    293 1 2 3 4 5 6
  # pressure ; 1013 14  bar
@@ -87,6 +90,10 @@ asciitext = """# these are really ugly data
 """
 
 
+def globalmodel(A, D, t, wave=0):
+    return A * np.exp(-wave ** 2 * D * t)
+
+
 # noinspection PyMethodMayBeStatic,PyMethodMayBeStatic,PyMethodParameters,PyMethodParameters
 class dataArrayTest(unittest.TestCase):
     """
@@ -233,7 +240,7 @@ class dataArrayTest(unittest.TestCase):
 
     def test_regrid(self):
         func = lambda x, y: x * (1 - x) * np.cos(np.pi * x) * np.sin(np.pi * y ** 2) ** 2 * 10
-        xz = js.parallel.randomPointsInCube(400, 0, 2) * 0.2 + 0.4
+        xz = js.formel.randomPointsInCube(400, 0, 2) * 0.2 + 0.4
         v = func(xz[:, 0], xz[:, 1])
         data = js.dA(np.stack([xz[:, 0], xz[:, 1], v], axis=0), XYeYeX=[0, 2, None, None, 1, None])
         xx = np.r_[0.42:0.58:0.04]
@@ -268,8 +275,10 @@ class dataListTest(unittest.TestCase):
 
         self.diff.fit(model, {'D': [0.1], 'A': 1}, {}, {'t': 'X', 'wave': 'q'}, condition=lambda a: a.X > 0.01,
                       output=False)
+        self.diffm.fit(globalmodel, {'D': [0.1], 'A': 1}, {}, {'t': 'X', 'wave': 'q'}, condition=lambda a: a.X > 0.01,
+                      output=False)
         # save should not save lastfit but list attributes
-        self.diff.save('testdiffreread.dat')
+        self.diff.save('testdiffreread.dat', fmt='%.8e')
         self.diffread = js.dL('testdiffreread.dat')
 
         self.pfdat = js.dL([js.dynamic.simpleDiffusion(q, np.r_[1:100:5], 1, 0.05) for q in np.r_[0.2:2:0.4]])
@@ -360,8 +369,8 @@ class dataListTest(unittest.TestCase):
         # test for length
         self.assertEqual(len(self.diff), len(self.diffread))
         # test common attributes
-        self.assertAlmostEqual(self.diff.D_err[-1], self.diffread.D_err[-1], delta=1e-12)
-        self.assertAlmostEqual(self.diff.D[-1], self.diffread.D[-1], delta=1e-12)
+        self.assertAlmostEqual(self.diff.D_err[-1], self.diffread.D_err[-1], delta=1e-9)
+        self.assertAlmostEqual(self.diff.D[-1], self.diffread.D[-1], delta=1e-9)
         # test if XYeYeX are retrieved correctly
         # here Z not present
         self.assertRaises(AttributeError, getattr, self.pfreread[-1], 'Z')
@@ -373,10 +382,13 @@ class dataListTest(unittest.TestCase):
         # normal pickle of dataList
         restoredm = pickle.loads(self.serializedm)
         nptest.assert_array_equal(restoredm[3].Y, self.diffm[3].Y)
-        # model was lambda an cannot be pickled so it is removed
+        # model was lambda or not top level and cannot be pickled, so it is removed
         restored = pickle.loads(self.serialized)
         nptest.assert_array_equal(restored[3].Y, self.diff[3].Y)
-        self.assertEqual(restored.model, 'removed during serialization')
+        # non top level model cannot be serialized
+        self.assertEqual(restored.model, 'removed during serialization, not pickabel function')
+        # top level model can be serialized
+        self.assertIn(b'jscatter.test', restoredm.model)
 
 
 def f(x, a, b, c, d):
@@ -391,7 +403,7 @@ class parallelTest(unittest.TestCase):
     def test_parallel(self):
         # loop over first argument, here x
         abcd = [1, 2, 3, 4]
-        res = js.parallel.doForList(f, looplist=np.arange(100), a=abcd[0], b=abcd[1], c=abcd[2], d=abcd[3])
+        res = js.formel.doForList(f, looplist=np.arange(100), a=abcd[0], b=abcd[1], c=abcd[2], d=abcd[3])
         self.assertEqual(res[0][0] + np.sum(abcd), res[0][1])
         self.assertEqual(res[-1][0] + np.sum(abcd), res[-1][1])
 
@@ -409,12 +421,12 @@ class parallelTest(unittest.TestCase):
         # without fortran we test several times python
         useFortran = js.parallel.useFortran
         if useFortran:
-            haltonfortran = js.parallel.haltonSequence(10, 2, 5)
+            haltonfortran = js.formel.haltonSequence(10, 2, 5)
             self.assertAlmostEqual(haltonfortran[1, 4], 0.370370370, 7)
             self.assertAlmostEqual(haltonfortran[0, 9], 0.9375, 7)
         # test python version
         js.parallel.useFortran = False
-        haltonpython = js.parallel.haltonSequence(10, 2, 5)
+        haltonpython = js.formel.haltonSequence(10, 2, 5)
         js.parallel.useFortran = useFortran  # reset
 
         if useFortran:
@@ -430,7 +442,7 @@ class fortranTest(unittest.TestCase):
         useFortran = js.cloudscattering.useFortran
         if useFortran:
             print('Using Fortran')
-            haltonfortran = js.parallel.haltonSequence(10, 2, 5)
+            haltonfortran = js.formel.haltonSequence(10, 2, 5)
             self.assertAlmostEqual(haltonfortran[1, 4], 0.370370370, 7)
             grid = js.sf.randomLattice([1, 1, 1], 5)
             cloudsphere = js.ff.cloudScattering(np.r_[0, 1], grid, relError=50)
@@ -738,6 +750,11 @@ class sasTest(unittest.TestCase):
         except OSError:
             pass
 
+    def test_creation(self):
+        imagearray = np.random.randn(256, 256) * 20 + 100
+        random = js.sas.sasImage(imagearray, 1, [100, 100], pixel_size=0.001, wavelength=2)
+        self.assertAlmostEqual(random.mean(), 100, delta=1)
+
     def test_sas(self):
         # peak position of AgBe reference
         AgBeref = js.sas.AgBeReference(self.q, 0.58378)
@@ -889,10 +906,10 @@ class plotTest(unittest.TestCase):
         self.data.fit(lambda x, A, a, B, p: A * np.sin(a * x + p) + B,
                       {'a': 1.2, 'p': [0], 'A': [1]}, {'B': 0}, {'x': 'X'}, output=False)
         try:
-            self.data.showlastErrPlot()
+            self.data.showlastErrPlot(size=(3, 2))
             self.data.errplot[1].yaxis(min=-0.02, max=0.02)
-            # data.savelastErrPlot('lastErrPlot_grace.png')
-            self.data.killErrPlot('lastErrPlot_kill_grace.png')
+            self.data.savelastErrPlot('lastErrPlot_grace.png', size=[3, 2], dpi=200)
+            self.data.killErrPlot()
 
             # simple plot
             pgrace = js.grace(1.3, 1)
@@ -916,19 +933,19 @@ class plotTest(unittest.TestCase):
         pmpl.Title('Sinusoidal fit')
         pmpl.Legend(charsize=0.7)
         # pmpl.tight_layout()
-        pmpl.Save('testimage_pmpl.png')
+        pmpl.Save('testimage_pmpl.png', size=[3, 2], dpi=200)
         pmpl.Close()
 
         js.usempl(1)
-        self.data.showlastErrPlot()
+        self.data.showlastErrPlot(size=(3, 2))
         self.data.errplot[1].Yaxis(min=-0.02, max=0.02)
-        # self.data.savelastErrPlot('lastErrPlot_mpl.png')
-        self.data.killErrPlot('lastErrPlot_kill_mpl.png')
+        self.data.savelastErrPlot('lastErrPlot_mpl.png', size=(3, 2), dpi=200)
+        self.data.killErrPlot()
 
     @classmethod
     def tearDown(self):
-        for file in ['testimage_pgrace.png', 'testimage_pmpl.png', 'lastplot.png', 'lastErrPlot_kill_grace.png',
-                     'lastErrPlot_kill_mpl.png', 'lastErrPlot.agr', 'lastErrPlot.png']:
+        for file in ['testimage_pgrace.png', 'testimage_pmpl.png', 'lastplot.png', 'lastErrPlot_grace.png',
+                     'lastErrPlot_mpl.png', 'lastErrPlot.agr', 'lastErrPlot.png']:
             try:
                 os.remove(file)
             except OSError:
@@ -937,27 +954,114 @@ class plotTest(unittest.TestCase):
 
     def test_simplePlot(self):
         if self.gracebattest:
-            refhexg = '0xfea04c15855ff3eae4cb18b13a756659c19a67e667c5ed912c72b6682491c00f'
+            refhexg = '0xfea00c15851ff3eae4ca18f13a756e58c19a676667c5ed913c72a4782591d08f'
             refgrace = js.formel.imageHash(image=refhexg, type='phash')
             pgracehash = js.formel.imageHash(image='testimage_pgrace.png', hashsize=16, type='phash')
+            print(1,refgrace.similarity(pgracehash),pgracehash.hex)
             self.assertLess(refgrace.similarity(pgracehash), 0.1)
 
-        refhexm = '0xfe876d1c8d7ab627e28d79973a66686ae1f987b90f0ca41cc93288e49c438e13'
+        refhexm = '0xeec22c0b97b7f3d27240395b2d37e48c81978fc3cd4a8c29da680fa0cc9f5a27'
         refmpl = js.formel.imageHash(image=refhexm, type='phash')
         pmplhash = js.formel.imageHash(image='testimage_pmpl.png', hashsize=16, type='phash')
-        self.assertLess(refmpl.similarity(pmplhash), 0.6)
+        print(2, refmpl.similarity(pmplhash),pmplhash.hex)
+        self.assertLess(refmpl.similarity(pmplhash), 0.2)
 
     def test_errPlot(self):
         if self.gracebattest:
-            refhexg = '0xadbf67f6c6c0b8e5f91e243bde862fc473c1c871c076d2dec278c643c006d109'
+            refhexg = '0xfefecd418d01b1aab07228fe98153b35ce0dd18fc3b3c7f09d848d188636869e'
             refgrace = js.formel.imageHash(image=refhexg, type='phash')
-            errgracehash = js.formel.imageHash(image='lastErrPlot_kill_grace.png', hashsize=16, type='phash')
-            self.assertLess(refgrace.similarity(errgracehash), 0.09)
+            errgracehash = js.formel.imageHash(image='lastErrPlot_grace.png', hashsize=16, type='phash')
+            print(3,refgrace.similarity(errgracehash),errgracehash.hex)
+            self.assertLess(refgrace.similarity(errgracehash), 0.1)
 
-        refhexm = '0xbeab4d478d0813b7f2f648c899593b39e76c9d8da19787f2b890c858c27ab218'
+        refhexm = '0xeeb92d44850bd3a5d2f03a565c782d2d472d97ad25c72dd2dc72017a6738c0f8'
         refmpl = js.formel.imageHash(image=refhexm, type='phash')
-        errmplhash = js.formel.imageHash(image='lastErrPlot_kill_mpl.png', hashsize=16, type='phash')
-        self.assertLess(refmpl.similarity(errmplhash), 0.15)
+        errmplhash = js.formel.imageHash(image='lastErrPlot_mpl.png', hashsize=16, type='phash')
+        print(4, refmpl.similarity(errmplhash),errmplhash.hex)
+        self.assertLess(refmpl.similarity(errmplhash), 0.5)
+
+
+class bioTest(unittest.TestCase):
+
+    setupdone = False
+
+    @classmethod
+    def setUp(self):
+        if self.setupdone: return
+        pdbfile = os.path.join(js.examples.datapath, 'arg61.pdb')
+
+        # switch of warnings because of MDAnalysis
+        warnings.simplefilter("ignore")
+
+        self.arg61 = js.bio.scatteringUniverse(pdbfile, addHydrogen=False)
+
+        # fast pARSING TEST THROUGH PDB2PQR TO ADD HYDROGEN
+        self.cln3 = js.bio.scatteringUniverse('3CLN', addHydrogen=True)
+
+        # xray and m scattering
+        self.Sx = js.bio.xscatIntUniv(self.arg61.atoms, output=0)  # volume will be calculated on this first call
+        self.Sxres = js.bio.xscatIntUniv(self.arg61.residues, output=0,refreshVolume=False)
+        self.Sn = js.bio.nscatIntUniv(self.arg61.atoms, output=0, refreshVolume=False)
+        # test explicit deuteration
+        self.arg61.select_atoms('name H').deuteration =1
+        self.Sndeut1 = js.bio.nscatIntUniv(self.arg61.atoms, output=0, refreshVolume=False)
+
+        # hydrodynamics + diffusion
+        self.hr = js.bio.hullRad(self.arg61.atoms, output=False)
+        Dtrans = self.hr['Dt'] * 1e2  # nm²/ps
+        Drot = self.hr['Dr'] * 1e-12  # 1/ps
+        self.Dq = js.bio.diffusionTRUnivTensor(self.arg61.atoms, Dtrans=Dtrans, Drot=Drot, refreshVolume=False,output=0)
+        self.Dqylm = js.bio.diffusionTRUnivYlm(self.arg61.atoms, Dtrans=Dtrans, Drot=Drot, refreshVolume=False)
+
+        # normal mode test
+        self.vnm = js.bio.vibNM(self.arg61.residues)
+
+        self.setupdone = True
+
+    @classmethod
+    def tearDown(self):
+        try:
+            os.remove('3CLN.pdb')
+            os.remove('3CLN_h.pqr')
+            os.remove('3CLN_h.pdb')
+        except FileNotFoundError:
+            pass
+        warnings.resetwarnings()
+
+    def test_formfactor(self):
+        # test formfactor values
+        nptest.assert_array_almost_equal(self.Sn.Y, [7.567e-07, 7.408e-07, 3.263e-07, 3.018e-07], 3)
+        nptest.assert_array_almost_equal(self.Sx.Y, [2.131e-05, 2.079e-05, 5.782e-06, 1.922e-06], 3)
+        nptest.assert_array_almost_equal(self.Sxres.Y, [2.131e-05, 2.078e-05, 5.708e-06, 1.774e-06], 3)
+        nptest.assert_array_almost_equal(self.Sndeut1.Y, [3.455e-06, 3.376e-06, 1.233e-06, 8.254e-07], 3)
+
+        # test basic properties like volume and corresponding contrast
+        self.assertAlmostEqual(self.Sn.SESVolume, 11.6598, 2)
+        self.assertAlmostEqual(self.Sn.edensitySolvent, 332.36, 2)
+        self.assertAlmostEqual(self.Sn.bcDensitySol, 0.0006363, 7)
+        self.assertAlmostEqual(self.Sn.sldExclSolvent, 0.0006363, 7)
+        self.assertAlmostEqual(self.Sn.massdensity, 1.343, 2)
+        self.assertAlmostEqual(self.Sn.RgInt, 2.622, 2)
+
+    def test_diffusion(self):
+        # diffusion constant
+        nptest.assert_array_almost_equal(self.Dqylm.Y, [0.000122, 0.000122, 0.000144, 0.000149], 6)
+        nptest.assert_array_almost_equal(self.Dq.Y,    [0.000122, 0.000126, 0.000344, 0.000418], 6)
+
+    def test_normalmode(self):
+        # normal mode eigenvalue first nontrivial
+        _ = self.vnm[6]
+        # zero eigenvalues
+        nptest.assert_almost_equal(self.vnm.eigenvalues[:6], 0, 5)
+        # first eigenvalues
+        nptest.assert_almost_equal(self.vnm.eigenvalues[6:11], [0.0099, 0.0099, 0.0469, 0.068, 0.0687], 3)
+        vnm6 = js.bio.intScatFuncPMode(self.vnm, 6, error=30, output=0, refreshVolume=False)
+        nptest.assert_array_almost_equal(vnm6.Y, [2.61e-21, 6.42e-16, 1.23e-10, 4.50e-10], 2)
+
+        # test normal mode options
+        nmresidues = js.bio.vibNM(self.arg61.residues).allatommode(6)
+        nmallatoms = js.bio.vibNM(self.arg61.residues[:10].atoms).allatommode(6)
+        nmaCaatoms = js.bio.vibNM(self.arg61.select_atoms('name CA')).allatommode(6)
 
 
 def suite():
@@ -975,6 +1079,7 @@ def suite():
     s.addTest(loader.loadTestsFromTestCase(test_imagehash.allimageHashTest))
     # s.addTest(loader.loadTestsFromTestCase(plotTest))
     s.addTest(loader.loadTestsFromTestCase(continTest))
+    s.addTest(loader.loadTestsFromTestCase(bioTest))
 
     return s
 
@@ -983,7 +1088,7 @@ def doTest2(verbosity=1):
     """Do only one test."""
     loader = unittest.TestLoader()
     suite = unittest.TestSuite()
-    suite.addTest(loader.loadTestsFromTestCase(sasTest))
+    suite.addTest(loader.loadTestsFromTestCase(plotTest))
     runner = unittest.TextTestRunner(verbosity=verbosity)
     runner.run(suite)
 
@@ -1005,5 +1110,72 @@ def doTest(verbosity=1):
     a = runner.run(suite())
     return a
 
+# Functions to run Examples from doc in module functions
+
+
+def getExamples(mod):
+    # get examples form jscatter module
+    examples = {}
+    for name, func in inspect.getmembers(mod, inspect.isfunction):
+        # get Examples section in numpydoc
+        examples[name] = []
+        example = FunctionDoc(func)['Examples']
+
+        if example == '':
+            # catch no example
+            continue
+
+        exampleline = -1
+        for i, line in enumerate(example+['.\n']):
+            if line.rstrip().endswith('::') and example[i+1].strip() == '':
+                # start of codeblock
+                examples[name].append([])
+                exampleline = 0
+            elif line.strip() == '':
+                if exampleline >0:
+                    examples[name][-1].append(line[1:])
+                    exampleline += 1
+            elif exampleline >= 0 and not line.startswith(' '):
+                # end of codeblock
+                exampleline = -1
+                # test if import jscatter line is there and remove not containing code blocks
+                if 'import jscatter' in '\n'.join(examples[name][-1]):
+                    pass
+                else:
+                    print(name)
+                    print(examples[name][-1])
+                    del examples[name][-1]
+
+            elif exampleline >= 0 and line.startswith(' '):
+                examples[name][-1].append(line[1:])
+                exampleline += 1
+
+    return examples
+
+
+def runExample(example, headless=True):
+    # run example catched using getExamples
+    if headless:
+        imports = [i for i in range(len(example)) if example[i].startswith('import jscatter')]
+        for i in imports[::-1]:
+            example.insert(i+1, 'js.headless(True)')
+
+    code ='\n'.join(example)+'\n'
+    # now do it
+    exec(code, {})
+
+    return
+
+
+def runModuleExamples(mod, headless=True):
+    for name, func in getExamples(mod).items():
+        for fi in func:
+            print('-'*40)
+            print(name)
+            print(fi)
+            runExample(fi, headless)
+    return
+
+
 if __name__ == '__main__':
     unittest.main(defaultTest=doTest)
